#include "gy85.h"
#include "i2c.h"

#include <util/delay.h>

/**
 * Initialize accelerometer.
 */
void GY85_init_acc(void)
{
	I2C_write(ADXL345, 0x31, 0x01);
	I2C_write(ADXL345, 0x2D, 0x08);
}

/**
 * Read X, Y, Z axis acceleration data.
 *
 * @param data_x - X-axis acceleration
 * @param data_y - Y-axis acceleration
 * @param data_z - Z-axis acceleration
 */
void GY85_read_acc(float *data_x, float *data_y, float *data_z)
{
	int buf[6];

	I2C_read_multiple(buf, ADXL345, 0x32, 6);

	*data_x = ((buf[1]) << 8) | buf[0];
	*data_y = ((buf[3]) << 8) | buf[2];
	*data_z = ((buf[5]) << 8) | buf[4];
}

/**
 * Initialize compass.
 */
void GY85_init_comp(void)
{
	I2C_write(HMC5883, 0x02, 0x00);
}

/**
 * Read X, Y, Z axis compass data.
 *
 * @param data_x - X-axis compass data
 * @param data_y - Y-axis compass data
 * @param data_z - Z-axis compass data
 */
void GY85_read_comp(float *data_x, float *data_y, float *data_z)
{
	int buf[6];

	I2C_read_multiple(buf, HMC5883, 0x03, 6);
	
	*data_x = buf[0] << 8 | buf[1];
	*data_y = buf[2] << 8 | buf[3];
	*data_z = buf[4] << 8 | buf[5];
}

static int g_offx = 0;
static int g_offy = 0;
static int g_offz = 0;

/**
 * Calibrate gyroscope.
 */
static void GY85_set_gyro(void)
{
	uint8_t i;
	float gp[4];
	
	int tmpx = 0;
	int tmpy = 0;
	int tmpz = 0;

	g_offx = 0;
	g_offy = 0;
	g_offz = 0;

	/* read 10 gyro samples and set offset */
	for (i = 0; i < 10; i++) {
		_delay_ms(10);

		GY85_read_gyro(&gp[0], &gp[1], &gp[2], &gp[3]);

		tmpx += gp[0];
		tmpy += gp[1];
		tmpz += gp[2];
	}

	g_offx = tmpx / 10;
	g_offy = tmpy / 10;
	g_offz = tmpz / 10;
}

/**
 * Initialize gyroscope.
 */
void GY85_init_gyro()
{
	I2C_write(ITG3200, 0x3E, 0x00);	
	I2C_write(ITG3200, 0x15, 0x07);
	I2C_write(ITG3200, 0x16, 0x1E);
	I2C_write(ITG3200, 0x17, 0x00);

	_delay_ms(10);

	GY85_set_gyro();
}

/**
 * Read X, Y, Z axis compass data.
 *
 * @param data_x - X-axis gyroscope data
 * @param data_y - Y-axis gyroscope data
 * @param data_z - Z-axis gyroscope data
 * @param t - temperature
 */
void GY85_read_gyro(float *data_x, float *data_y, float *data_z, float *t)
{   
	uint8_t buf[8];

	I2C_read_multiple(buf, ITG3200, 0x1B, 8);

	*data_x = ((buf[4] << 8) | buf[5]) - g_offx;
	*data_y = ((buf[2] << 8) | buf[3]) - g_offy;
	*data_z = ((buf[6] << 8) | buf[7]) - g_offz;
	*t = ((buf[0] << 8) | buf[1]);
}
